"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events = require("events");
const LiskClient_1 = require("./LiskClient");
const _ = require("underscore");
var LiskPeerEvent;
(function (LiskPeerEvent) {
    LiskPeerEvent["statusUpdated"] = "STATUS_UPDATED";
    LiskPeerEvent["peersUpdated"] = "PEERS_UPDATED";
    LiskPeerEvent["nodeStuck"] = "NODE_STUCK";
})(LiskPeerEvent = exports.LiskPeerEvent || (exports.LiskPeerEvent = {}));
var PeerState;
(function (PeerState) {
    PeerState[PeerState["ONLINE"] = 0] = "ONLINE";
    PeerState[PeerState["OFFLINE"] = 1] = "OFFLINE";
})(PeerState = exports.PeerState || (exports.PeerState = {}));
/***
 * LiskPeer is a wrapper for LiskClient that automatically updates the node status and keeps track of the connection
 * and checks whether the node is sane (not stuck)
 */
class LiskPeer extends events.EventEmitter {
    constructor(_options, ownNonce, ownVersion) {
        super();
        this._options = _options;
        this.ownNonce = ownNonce;
        this.ownVersion = ownVersion;
        this.peers = [];
        this.knownBy = 0;
        this._lastHeightUpdate = 0;
        this._stuck = false;
        this._state = PeerState.OFFLINE;
        this._httpActive = false;
        this._wsServerConnected = false;
        this.client = new LiskClient_1.LiskClient(_options.ip, _options.wsPort, _options.httpPort, {
            nonce: ownNonce,
            nethash: _options.nethash,
            version: ownVersion,
            os: "linux",
            height: 500,
            wsPort: _options.ownWSPort,
            httpPort: _options.ownHttpPort,
        });
        // Check whether client supports HTTP
        this.client.http
            .getNodeStatus()
            .then(() => (this._httpActive = true))
            .catch(() => { });
        // Connect via WebSocket
        this.client.connect(() => this.onClientConnect(), () => this.onClientDisconnect(), error => this.onClientError(error));
        // Schedule status updates
        this.statusUpdateInterval = setInterval(() => this.updateStatus(), 2000);
    }
    get state() {
        return this._state;
    }
    get status() {
        return this._status;
    }
    get httpActive() {
        return this._httpActive;
    }
    get wsServerConnected() {
        return this._wsServerConnected;
    }
    get options() {
        return this._options;
    }
    /***
     * Handles new NodeStatus received from the Peer
     * Updates sync status and triggers events in case the node is stuck
     * @param {NodeStatus} status
     */
    handleStatusUpdate(status) {
        if (!this._status || status.height > this._status.height) {
            this._lastHeightUpdate = Date.now();
            this._stuck = false;
        }
        else if (!this._stuck && Date.now() - this._lastHeightUpdate > 20000) {
            this._stuck = true;
            this.emit(LiskPeerEvent.nodeStuck);
        }
        // Apply new status
        if (!this._status)
            this._status = status;
        this._status = Object.assign(this._status, status);
        this._options.nonce = status.nonce;
        // Emit the status update
        this.emit(LiskPeerEvent.statusUpdated, status);
    }
    /***
     * Set the connection status of the websocket connection for this peer
     * @param {boolean} connected
     */
    setWebsocketServerConnected(connected) {
        this._wsServerConnected = connected;
    }
    /***
     * Destroy the peer and close/destroy the associated socket
     * This does not close the incoming socket connection
     */
    destroy() {
        clearInterval(this.statusUpdateInterval);
        this.client.destroy();
    }
    requestBlocks() {
        //if (this.httpActive) {
        //    this.client.getBlocksHTTP().then((blockData) => {
        //
        //    })
        //} else {
        this.client.getBlocks().then(blockData => {
            let filteredBlocks = _.uniq(blockData.blocks, entry => entry.b_id);
        });
        //}
    }
    onClientConnect() {
        // console.debug(`connected to ${this._options.ip}:${this._options.wsPort}`);
        this._state = PeerState.ONLINE;
    }
    onClientDisconnect() {
        // console.debug(`disconnected from ${this._options.ip}:${this._options.wsPort}`);
        this._state = PeerState.OFFLINE;
    }
    onClientError(error) {
        console.error(`connection error from ${this._options.ip}:${this._options.wsPort}: ${error}`);
    }
    /***
     * Trigger a status update
     * Updates the node status, connected peers
     */
    updateStatus() {
        if (this._state != PeerState.ONLINE)
            return;
        this.client
            .getStatus()
            .then(status => this.handleStatusUpdate(status))
            .then(() => this.client.getPeers())
            .then(res => {
            this.peers = res.peers;
            this.emit(LiskPeerEvent.peersUpdated, res.peers);
        })
            .catch(err => console.warn(`could not update status of ${this._options.ip}:${this._options.wsPort}: ${err}`));
    }
}
exports.LiskPeer = LiskPeer;
//# sourceMappingURL=LiskPeer.js.map